//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Program        : Simple 2d Array Demonstations
//
// Author         : Richard E. Pattis
//                  Computer Science Department
//                  Carnegie Mellon University
//                  5000 Forbes Avenue
//                  Pittsburgh, PA 15213-3891
//                  e-mail: pattis@cs.cmu.edu
//
// Maintainer     : Author
//
//
// Description:
//
//   This program shows how to declare, initialize, and manipulate a simple
// 2d array of int values using a variety of nested for loops. It would be
// an excellent idea to hand simulate a few of them to see how the for
// control structure and 2d array data structure work together
//
// Known Bugs     : None
//
// Future Plans   : None
//
// Program History:
//   6/07/01: R. Pattis - Operational for 15-100
//   9/25/01: R. Pattis - Extended and commented further
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////


import edu.cmu.cs.pattis.cs151xx.Prompt;
import edu.cmu.cs.pattis.cs151xx.TypedBufferReader;

import java.io.IOException;



public class Application {

	public static void main(String[] args)
	{
	  try {
	  
	    //Prompt user for rows and columns of 2d array and construct an
	    //  object of that size (using a special array constructor)
      //Try entering a negative length and see what happens
	    int     rows = Prompt.forInt("Enter # of rows in the array");
	    int     cols = Prompt.forInt("Enter # of cols in the array");
      int[][] a    = new int[rows][cols];
      System.out.println();
      
      //The array is declared to have row indexes from 0 to a.length-1
      //The array is declared to have col indexes from 0 to a[0].length-1,
      //  assuming a.length is at least 1
      
      
      //Prompt the user to store a value at each index in the object
      //  (alternatively, switch comment and store random values)
      //Note the standard use of nested for loops: the outer one for
      //  the row, the inner one for the column
      //This is called row-major order (processes each row)
      for (int r=0; r<a.length; r++)
        for (int c=0; c<a[0].length; c++)
          //a[r][c] = Prompt.forInt("Enter value for ["+r+"]["+c+"]");
          a[r][c] = (int)(Math.random()*100);
       
      
  
      //Print out what is stored in each index in the array 
      System.out.println("Array contains ("+a.length+","+a[0].length+"): ");  
      for (int r=0; r<a.length; r++) {
        for (int c=0; c<a[0].length; c++)
          System.out.print(a[r][c]+" ");
        System.out.println();
      }
      System.out.println();
      
   
      
      //Compute/print the sum of all the array values  
      int sum = 0;
      for (int r=0; r<a.length; r++)
        for (int c=0; c<a[0].length; c++)
        sum += a[r][c];
      System.out.println("Sum = " + sum);
      
  
      
      //Compute/print the maximum of all the array values  
      int max = a[0][0];
      for (int r=0; r<a.length; r++)
        for (int c=0; c<a[0].length; c++)
	        if (a[r][c]>max)
	          max = a[r][c];
      System.out.println("Max = " + max);
      //We could also set max = Integer.MIN_VALUE; in either case why
      //  must we start both loops at 0? This is a bit different than the
      //  1d array way
      
      
      
     
      //Compute/print indexes of value to search for
      //Note the labelled loop and labelled break; otherwise the
      //  break would break just out of the inner loop
      //Normally this feature is not needed, because the code
      //  would be in a method, which would use a return
      int toSearch = Prompt.forInt("Enter value to search for");
      int foundR = -1, foundC = -1; 
      outer: for (int r=0; r<a.length; r++)
        for (int c=0; c<a[0].length; c++)
          if (a[r][c] == toSearch) {
            foundR = r;     //We know the answer
            foundC = c;
            break outer;          //Don't even bother looking at any more values
          }
      if (foundR != -1 && foundC != -1)
        System.out.println("Found at = ["+foundR+","+foundC+"]");
      else
        System.out.println("Not found");
      
     
      
      //Compute/print whether the array values get bigger (when scanning
      //  in row-major order)  
      boolean increasingOrder = true;
      int     previous = Integer.MIN_VALUE;
      outer: for (int r=0; r<a.length; r++)
        for (int c=0; c<a[0].length; c++) {
          if (previous > a[r][c]) {
            increasingOrder = false;
            break outer;
          }
          previous = a[r][c];
        }
      System.out.println("Increasing = " + increasingOrder);
      


      //We can easily load a file into an array, but we need to know
      //  how big to make the Matrix (unlike 1d arrays, we cannot just
      //  grow it because we need to know where each value goes.
      //Try the input.txt and badinputX.txt files when prompted here
      //  badinput.txt has all ints, but too many of them for b
      TypedBufferReader tbr = new TypedBufferReader("Enter name for file of int");
      rows = tbr.readInt();
      cols = tbr.readInt();
      a = new int[rows][cols];
      try {
      for (int r=0; r<a.length; r++)
        for (int c=0; c<a[0].length; c++) {
          a[r][c] = tbr.readInt();
        }
      }catch (IOException ioe)
        {System.out.println("Not enough values in file"); throw ioe;}
      tbr.close();
      
      
      System.out.println("Array contains ("+a.length+","+a[0].length+"): ");  
      for (int r=0; r<a.length; r++) {
        for (int c=0; c<a[0].length; c++)
          System.out.print(a[r][c]+" ");
        System.out.println();
      }
      System.out.println();


    }catch (Exception e) {
      e.printStackTrace();
      System.out.println("main method in Application class terminating");
      System.exit(0);  
    }

	}


}
